from rest_framework import serializers
from rest_framework.serializers import ValidationError
from django.core.validators import RegexValidator

class ChoicesField(serializers.Field):
    def __init__(self, choices, **kwargs):
        self._choices = choices

        if 'default_value' in kwargs:
            self.default_value = kwargs.pop("default_value")

        super(ChoicesField, self).__init__(**kwargs)

    def to_representation(self, value):        
        display_value = self.get_display_value_or_none(value)
        if display_value != None:
            return display_value
        else:
            try:
                return self.default_value[1]
            except AttributeError:
                raise ValidationError("{} is not a valid value.".format(value))

    def to_internal_value(self, value):
        internal_value = self.get_internal_value_or_none(value)
        if internal_value != None:
            return internal_value
        else:
            try:
                return self.default_value[0]
            except AttributeError:
                raise ValidationError("{} is not a valid value.".format(value))
        

    def get_display_value_or_none(self, value):
        for choice in self._choices:
            if choice[0] == value or choice[1] == value:
                return choice[1]
        return None

    def get_internal_value_or_none(self, value):
        for choice in self._choices:
            if choice[0] == value or choice[1] == value:
                return choice[0]
        return None


class UUIDField:

    @ staticmethod
    def getField(error_message="Invalid UUID"):
        return serializers.CharField(
            max_length=22, validators=[RegexValidator(regex=r"^[a-zA-Z0-9]{22}$", message=error_message)])



from email_validator import validate_email as email_validator, EmailNotValidError
from rest_framework.serializers import ValidationError


def get_protected_email(
    s): return '@'.join(x[0]+'*'*(len(x)-2)+x[-1] for x in s.split('@'))


def validate_email(email):
    try:
        # Validate.
        email_validator(email)
    except EmailNotValidError as e:
        # email is not valid, exception message is human-readable
        raise ValidationError('{} is not a valid email'.format(email))



from .lambda_client import AbstractLambdaClient
from .normalize import map_values


class AddressValidationClient(AbstractLambdaClient):

    IS_VALID_ADDRESS = "is_valid_address"
    NORMALIZE_ADDRESS = 'normalize_address'

    valid_triggers = [IS_VALID_ADDRESS, NORMALIZE_ADDRESS]

    def __init__(self):
        super().__init__("us-east-2", "AddressValidation")

    def is_valid_address(self, *, address_line_one, postal_code, country="US", city=None, state=None, address_line_two=None):
        body = {
            "addressLineOne": address_line_one,
            "postalCode": postal_code,
            "country": country
        }

        if city != None:
            body['city'] = city

        if state != None:
            body['state'] = state

        if address_line_two != None:
            body['addressLineTwo'] = address_line_two

        try:
            super().invoke(self.IS_VALID_ADDRESS, body)
            return True
        except:
            return False

    def normalize_address(self, *, address_line_one, postal_code, country="US", city=None, state=None, address_line_two=None):
        body = {
            "addressLineOne": address_line_one,
            "postalCode": postal_code,
            "country": country
        }

        if city != None:
            body['city'] = city

        if state != None:
            body['state'] = state

        if address_line_two != None:
            body['addressLineTwo'] = address_line_two

        response = super().invoke(self.NORMALIZE_ADDRESS, body)

        return map_values(response['body'], {
            "address_line_one": "address_one",
            "address_line_two": "address_two",
            "state_code": "state",
            "country_code": "country",
            "geo": "geocode"
        }
        )



from .lambda_client import AbstractLambdaClient
from rest_framework.serializers import ValidationError
from .cache_manager import CacheManager
from bartab_core.validate import is_valid_uuid
class UserInfoCacheManager:    
    USERNAME_BY_UNIQUE_IDENTIFER_KEY_NAME = "user_info_username_by_unique_identifier_map"

    USER_INFO_BY_USERNAME_KEY_NAME = 'user_info_user_info_map_by_username_map'

    def __init__(self):
        self.cache_manager = CacheManager.get_instance()

    def get_user_info_or_none(self, username:str):
        user_info_map_by_username = self.cache_manager.set_or_get(
            self.USER_INFO_BY_USERNAME_KEY_NAME, {})

        return None if username not in user_info_map_by_username else user_info_map_by_username[username]

    def get_username_or_none(self, unique_user_identifier: str):
        username_by_unique_identifier = self.cache_manager.set_or_get(
            self.USERNAME_BY_UNIQUE_IDENTIFER_KEY_NAME, {})

        return None if unique_user_identifier not in username_by_unique_identifier else username_by_unique_identifier[unique_user_identifier]

    def set_user_info(self, user_info):
        user_info_map_by_username = self.cache_manager.set_or_get(
            self.USER_INFO_BY_USERNAME_KEY_NAME, {})
        
        user_info_map_by_username[user_info['username']] = user_info

        self.cache_manager.set(
            self.USER_INFO_BY_USERNAME_KEY_NAME, user_info_map_by_username
        )

    def set_username_map(self, unique_user_identifier, username):
        username_by_unique_identifier = self.cache_manager.set_or_get(
            self.USERNAME_BY_UNIQUE_IDENTIFER_KEY_NAME, {})

        username_by_unique_identifier[unique_user_identifier] = username

        self.cache_manager.set(
            self.USERNAME_BY_UNIQUE_IDENTIFER_KEY_NAME, username_by_unique_identifier
        )


class UserInfoClient(AbstractLambdaClient):

    USER_INFO = 'user_info'
    USERNAME = 'username'
    GET_USER_POOL_INFO = 'get_user_pool_info'
    GET_USER_ATTRIBUTE_INFO = 'get_user_attribute_info'

    valid_triggers = [USERNAME, USER_INFO,
                      GET_USER_POOL_INFO, GET_USER_ATTRIBUTE_INFO]

    def __init__(self):
        super().__init__("us-east-2", "UserInfo", True, True)
        self.user_info_cache_manager = UserInfoCacheManager()

    def get_user_info(self, unique_user_identifier, use_cache: bool = True):
        if use_cache:
            if is_valid_uuid(unique_user_identifier):
                username = unique_user_identifier
            else:
                username = self.get_username(unique_user_identifier, True)

            user_info = self.user_info_cache_manager.get_user_info_or_none(
                username)
            
            if user_info != None:
                return user_info

        user_info = super().invoke(self.USER_INFO, {
            "value": unique_user_identifier
        })['body']

        self.user_info_cache_manager.set_user_info(user_info)
        
        return user_info

    def get_username(self, unique_user_identifier, use_cache: bool = True):
        if use_cache:
            username = self.user_info_cache_manager.get_username_or_none(
                unique_user_identifier)
            if username != None:
                return username

        username = super().invoke(self.USERNAME, {
            "value": unique_user_identifier
        })['body']['value']

        self.user_info_cache_manager.set_username_map(
            unique_user_identifier, username)
        
        return username

    def get_user_pool_info(self):
        return super().invoke(self.GET_USER_POOL_INFO)['body']

    def get_user_attribute_info(self):
        return super().invoke(self.GET_USER_ATTRIBUTE_INFO)['body']

    def get_user_display_name(self, unique_user_identifier_or_user_info, fail_on_unknown: bool = False, use_cache: bool = True):
        name = ''

        if isinstance(unique_user_identifier_or_user_info, dict):
            user_info = unique_user_identifier_or_user_info
        else:
            user_info = self.get_user_info(
                unique_user_identifier_or_user_info, use_cache)

        if 'first_name' in user_info and len(user_info['first_name']) > 0:
            name = user_info['first_name']

        if 'last_name' in user_info and len(user_info['last_name']) > 0:
            if len(name) > 0:
                name = name + " "

            name = name + user_info['last_name']

        if name != '':
            return name
        elif fail_on_unknown:
            raise ValidationError("User has invalid name")
        else:
            return "Unknown User"

    def is_staff(self, unique_user_identifier, use_cache: bool = True):
        return self.get_user_info(unique_user_identifier, use_cache)['is_staff']

    def is_sales(self, unique_user_identifier, use_cache: bool = True):
        user_info = self.get_user_info(unique_user_identifier, use_cache)
        
        return user_info['is_staff'] or user_info['is_sales']



from django.conf import settings
from .paths import get_relative_path
from .json import JsonFile
import datetime
import pytz
import datetime


class TimeZoneData:

    TIME_ZONE_GENERATED_DATA_PATH = get_relative_path(
        '../data/generated/timezones.json', __file__)

    @staticmethod
    def data():
        timezone_data_file = JsonFile(
            TimeZoneData.TIME_ZONE_GENERATED_DATA_PATH)

        if not timezone_data_file.is_valid():
            TimeZoneData.__generate_timezone_data()

        return timezone_data_file.data()

    @staticmethod
    def __generate_timezone_data():
        timezones_by_offset_file = JsonFile(
            TimeZoneData.TIME_ZONE_GENERATED_DATA_PATH)

        timezone_data = {
            'timezones_by_offset': {},
            'offset_by_timezone_name': {},
            'us_time_zones_by_offset': {}
        }

        for timezone in pytz.common_timezones:
            timezone_now = datetime.datetime.now(pytz.timezone(timezone))
            offset_time_in_hours = timezone_now.utcoffset().total_seconds()/60/60

            if offset_time_in_hours in timezone_data['timezones_by_offset']:
                timezone_data['timezones_by_offset'][offset_time_in_hours].append(
                    timezone)
            else:
                timezone_data['timezones_by_offset'][offset_time_in_hours] = [
                    timezone]

            if 'US/' in timezone:
                timezone_data['us_time_zones_by_offset'][offset_time_in_hours] = timezone

            timezone_data['offset_by_timezone_name'][timezone] = offset_time_in_hours

        timezones_by_offset_file.set_json(timezone_data)


def is_same_day(date_time_one, date_time_two, date_time_format=None):
    if date_time_format == None:
        date_time_format = settings.DATETIME_FORMAT

    if isinstance(date_time_one, str):
        date_time_one = datetime.datetime.strptime(
            date_time_one, date_time_format)
    elif not isinstance(date_time_one, datetime.datetime):
        raise ValueError("Invalid date type")

    if isinstance(date_time_two, str):
        date_time_two = datetime.datetime.strptime(
            date_time_two, date_time_format)
    elif not isinstance(date_time_two, datetime.datetime):
        raise ValueError("Invalid date type")

    return date_time_one.date() == date_time_two.date()


def is_today(date_time, date_time_format=None):
    if date_time_format == None:
        date_time_format = settings.DATETIME_FORMAT
    
    return is_same_day(date_time, datetime.datetime.now(), date_time_format)


def is_future(date_time, date_time_format=None):
    if date_time_format == None:
        date_time_format = settings.DATETIME_FORMAT
    
    if isinstance(date_time, str):
        return datetime.datetime.now() < datetime.datetime.strptime(date_time, date_time_format)
    elif isinstance(date_time, datetime.datetime):
        return datetime.datetime.now() < date_time
    elif isinstance(date_time, datetime.date):
        return datetime.datetime.now() < datetime.combine(date_time, datetime.min.time())
    else:
        raise ValueError("Invalid date type")


def is_past(date_time, date_time_format=None):
    if date_time_format == None:
        date_time_format = settings.DATETIME_FORMAT
    
    return not is_future(date_time, date_time_format)



from . import api_config as microservices_config
from rest_framework.exceptions import APIException
from django.core.exceptions import ImproperlyConfigured
from django.conf import settings
from rest_framework import status
from http.client import responses
import requests


class MicroserviceServiceUnavailable(APIException):
    status_code = 502
    default_code = 'service_unavailable'


class MicroserviceResponse:
    def __init__(self,
                 response,
                 exception,
                 ):
        if status.is_server_error(response.status_code):
            raise exception
        else:
            for key in response.__dict__:
                setattr(self, key, response.__dict__[key])

            self.response = response

    def json(self):
        try:
            return self.response.json()
        except:
            if status.is_success(self.response.status_code):
                return None
            else:
                return {'detail': responses[self.status_code]}

    def close(self):
        self.response.close()

    def iter_content(self):
        return self.response.iter_content()

    def iter_lines(self):
        return self.response.iter_lines()

    def raise_for_status(self):
        return self.response.raise_for_status()


class AbstractMicroservice:

    GET = 'get'
    POST = 'post'
    PUT = 'put'
    DELETE = 'delete'

    def __init__(self, *,
                 url_prefix,
                 localhost_port,
                 service_name,
                 exception,
                 api_version,
                 service_description
                 ):

        self.service_description = service_description

        try:
            self.api_key = microservices_config.get_api_key(service_name)
            self.valid_connection = True
        except ImproperlyConfigured:
            self.valid_connection = False
            
            if settings.DEBUG:
                self.invalid_configuration_error_message = microservices_config.get_invalid_configuration_error_message(
                    service_name)
            else:
                self.invalid_configuration_error_message = "{} service is currently unaviable, please try again later.".format(
                    self.service_description).capitalize()

        self.endpoint = microservices_config.get_endpoint(
            url_prefix=url_prefix,
            api_version=api_version,
            localhost_port=localhost_port
        )

        self.exception = exception

    def __make_endpoint(self, path, request_type):
        if request_type == self.POST and len(path) > 0 and path[-1] != '/':
            path = "{}/".format(path)

        return '{0}/{1}'.format(self.endpoint, path)

    def __make_request(self,
                       path,
                       request,
                       request_type):

        if not self.valid_connection:
            raise MicroserviceServiceUnavailable(self.invalid_configuration_error_message)

        headers = {
            'content-type': 'application/json',
            settings.API_KEY_PARAM_NAME: self.api_key
        }

        if request != None:
            data = request.data
            params = request.query_params

            bearer_header = request.META.get('HTTP_AUTHORIZATION')
            if bearer_header != None and bearer_header != '':
                headers['Authorization'] = bearer_header
        else:
            data = None
            params = None

        if request_type == self.POST:
            response = requests.post(
                self.__make_endpoint(path, self.POST),
                json=data,
                params=params,
                headers=headers
            )
        elif request_type == self.PUT:
            response = requests.put(
                self.__make_endpoint(path, self.PUT),
                json=data,
                params=params,
                headers=headers
            )
        elif request_type == self.DELETE:
            response = requests.delete(
                self.__make_endpoint(path, self.DELETE),
                json=data,
                params=params,
                headers=headers
            )
        else:
            response = requests.get(
                self.__make_endpoint(path, self.GET),
                json=data,
                params=params,
                headers=headers
            )

        return MicroserviceResponse(response, self.exception)

    def get(self, path, request=None):
        return self.__make_request(path, request, self.GET)

    def post(self, path, request=None):
        return self.__make_request(path, request, self.POST)

    def put(self, path, request=None):
        return self.__make_request(path, request, self.PUT)

    def delete(self, path, request=None):
        return self.__make_request(path, request, self.DELETE)

    class Meta:
        abstract = True



import phonenumbers
from rest_framework.serializers import ValidationError


def validate_phone_number(phone_number):
    try:
        phone_number_obj = phonenumbers.parse(phone_number, None)
        if not phonenumbers.is_possible_number(phone_number_obj) or not phonenumbers.is_valid_number(phone_number_obj):
            raise ValidationError(
                {'phone_number': [phone_number+' is not a valid phone number']})
    except ValidationError as exp:
        raise exp
    except phonenumbers.NumberParseException as exp:
        raise ValidationError(
            {'phone_number': ['Phone number invalid, format: +11234567890']})
    except Exception as exp:
        raise exp


def is_valid_phone_number(phone_number: str) -> bool:
    try:
        validate_phone_number(phone_number)
        return True
    except:
        return False


def get_protected_phone_number(phone_number: str) -> str:
    if is_valid_phone_number(phone_number):
        len_phone_number_no_nation = len(phone_number)
        phone_number_end = phone_number[len_phone_number_no_nation -
                                                  4: len_phone_number_no_nation]

        return f"+1******{phone_number_end}"
    raise ValueError("Invalid phone number")



from django.core.cache import cache
from .list import normalize_list


class CacheManager():
    __instance__ = None
    default_populate_values = {}

    def __init__(self):
        if CacheManager.__instance__ is None:
            super().__init__()
            CacheManager.__instance__ = self
        else:
            raise ValueError("You cannot create another CacheManager class")

    @ staticmethod
    def get_instance():
        """ Static method to fetch the current instance.
        """
        if not CacheManager.__instance__:
            CacheManager()
        return CacheManager.__instance__

    def add_default_population_value(self, value_name, populate_function, param_values=[]):
        param_values = normalize_list(param_values)

        self.default_populate_values[value_name] = {
            "function": populate_function, "param_values": param_values}

        self.populate_value(value_name)

    def reset_default_values(self):
        self.default_populate_values = {}

    def reset_cache(self):
        cache.clear()
        self.populate_values()

    def full_reset(self):
        self.reset_default_values()
        self.reset_cache()

    def populate_value(self, value_name):
        if value_name not in self.default_populate_values:
            raise ValueError(f'{value_name} is not a managed cache value')
        else:
            managed_cache_value = self.default_populate_values[value_name]

            param_values = managed_cache_value['param_values']
            populate_function = managed_cache_value['function']

            if(len(param_values) == 0):
                self.set(value_name, populate_function())
            else:
                self.set(value_name, populate_function(*param_values))

    def populate_values(self):
        for value_name in self.default_populate_values:
            self.populate_value(value_name)

    def set(self, value_name, value):
        cache.set(value_name.replace(" ", '-'), value)

    def get(self, value_name):
        return cache.get(value_name.replace(" ", '-'))

    def set_or_get(self, value_name, default_value):
        if self.get(value_name) == None:
            self.set(value_name, default_value)

        return self.get(value_name)



import boto3
from rest_framework.serializers import ValidationError
import json
from .normalize import map_to_slug_keys
from rest_framework import status


class AbstractLambdaClient:

    LIST_SUPPORTED_TRIGGERS = "supported_triggers"

    def __init__(self, region, function_name, has_local=False, has_qa=False):
        from django.conf import settings

        self.function_name = function_name
        self.client = boto3.client(
            'lambda',
            region_name=region,
            aws_access_key_id=settings.AWS_ACCESS_KEY_ID,
            aws_secret_access_key=settings.AWS_SECRET_ACCESS_KEY
        )

        self.has_local = has_local
        self.has_qa = has_qa

        if hasattr(self, 'valid_triggers'):
            self.valid_triggers.append(self.LIST_SUPPORTED_TRIGGERS)
        else:
            self.valid_triggers = [self.LIST_SUPPORTED_TRIGGERS]

    def invoke(self, trigger, body={}, return_slug_values=True):
        from django.conf import settings

        if trigger not in self.valid_triggers:
            raise ValidationError({
                "trigger_source": [
                    'Trigger {0} not supported, supported triggers: {1}'.format(
                        trigger, ", ".join(self.valid_triggers))
                ]
            }
            )

        function_name = self.function_name
        if not settings.IS_PRODUCTION:
            if settings.IS_QA and self.has_qa:
                function_name = f"{function_name}QA"
            elif not settings.IS_DEVELOPMENT and not settings.IS_QA and self.has_local:
                function_name = f"{function_name}Local"
            else:
                function_name = f"{function_name}Dev"

        body['triggerSource'] = trigger

        response = self.client.invoke(
            FunctionName=function_name,
            InvocationType='RequestResponse',
            LogType='None',
            Payload=json.dumps(body).encode('utf-8')
        )

        response_payload = json.loads(
            response['Payload'].read().decode("utf-8"))

        if 'statusCode' in response_payload:
            status_code = response_payload['statusCode']
            if return_slug_values:
                response_payload = map_to_slug_keys(response_payload)

            if status.is_success(status_code):
                return response_payload
            elif status.is_client_error(status_code):
                raise ValidationError(response_payload['body'])
            else:
                if 'detail' in response_payload['body']:
                    raise Exception(response_payload['body']['detail'])
                else:
                    raise Exception("Lambda function error occurred")
        elif 'errorMessage' in response_payload:
            raise Exception(response_payload['errorMessage'])
        else:
            raise Exception("Unknown lambda response")

    class Meta:
        abstract = True



from .string import camel_case_to_slug, display
from .list import normalize_list


def map_to_slug_keys(map_value):
    new_map_value = {}
    for key in map_value:
        if isinstance(map_value[key], dict):
            new_map_value[camel_case_to_slug(
                key)] = map_to_slug_keys(map_value[key])
        else:
            new_map_value[camel_case_to_slug(key)] = map_value[key]
    return new_map_value


def map_values(
        map_value: dict,
        keys_to_exchange: dict,
        override_existing: bool = True) -> dict:

    for existing_key in keys_to_exchange:
        if existing_key in map_value:
            new_key = keys_to_exchange[existing_key]

            if new_key not in map_value or override_existing:
                map_value[new_key] = map_value[existing_key]
                del map_value[existing_key]

    return map_value


class NormalizedBool:
    VALID_EXACT_TRUE_STRING = 'TRUE'
    VALID_EXACT_FALSE_STRING = 'FALSE'

    VALID_TRUE_STRINGS = ['T', VALID_EXACT_TRUE_STRING, 'YES', 'Y', '1']
    VALID_FALSE_STRINGS = [
        'F', VALID_EXACT_FALSE_STRING, 'NO', 'N', '0', 'NONE']

    def __init__(self, value, default_exact_equivalency=False):
        self.__sent_value = value
        value = str(value).upper()
        self.__is_valid = True
        self.__default_exact_equivalency = default_exact_equivalency

        self.__is_valid_exact = (
            value == self.VALID_EXACT_TRUE_STRING or value == self.VALID_EXACT_FALSE_STRING)

        if value in self.VALID_TRUE_STRINGS:
            self.__bool_value = True
        elif value in self.VALID_FALSE_STRINGS:
            self.__bool_value = False
        else:
            self.__is_valid = False

    def get_error_message(self, exact=None):
        exact = exact if exact != None else self.__default_exact_equivalency

        if exact and not self.__is_valid_exact:
            return "Strict equivalency is being enforced for this method. {} is not True or False".format(self.__sent_value)
        elif not exact and not self.__is_valid:
            return "{} is not a valid value. Valid values are: {}".format(self.__sent_value, ", ".join(
                (
                    self.VALID_TRUE_STRINGS + self.VALID_FALSE_STRINGS
                )
            ))
        else:
            return None

    def is_valid(self, exact=None):
        exact = exact if exact != None else self.__default_exact_equivalency

        return self.__is_valid_exact if exact else self.__is_valid

    def get(self, exact=None):
        exact = exact if exact != None else self.__default_exact_equivalency

        if not self.is_valid(exact):
            raise ValueError({"bool": self.get_error_message(exact)})

        return self.__bool_value


class NomalizedString:

    def __init__(self, value, valid_values, default_exact_equivalency=False):
        self.sent_value = value
        self.__valid_values = normalize_list(valid_values)
        self.__default_exact_equivalency = default_exact_equivalency

    def is_valid(self, exact=None):
        exact = exact if exact != None else self.__default_exact_equivalency

        if exact:
            return self.sent_value in self.__valid_values
        else:
            return self.sent_value.lower() in list(map(lambda s: s.lower(), self.__valid_values))

    def get_valid_value(self):
        for valid_value in self.__valid_values:
            if valid_value.lower() == self.sent_value.lower():
                return valid_value
        raise ValueError(f"{self.sent_value} is not a valid value")



import uuid
import shortuuid
from rest_framework.test import APIRequestFactory
from django.urls.resolvers import ResolverMatch
from django.urls import reverse

def generate_uuid():
    return str(uuid.uuid4())

def generate_short_uuid():
    return str(shortuuid.uuid())


def create_uuid(long: bool = False, value: str = None):
    if isinstance(value, bool) and not value:
        return None
    elif value == None or (isinstance(value, bool) and value):
        return generate_uuid() if long else generate_short_uuid()
    else:
        return value
    
class MockAuthorizedRequest:

    GET ='get'
    POST = 'post'
    DELETE = 'delete'
    PUT = 'put'

    format='json'

    factory = APIRequestFactory()

    def __init__(self, bartab_user: str = generate_uuid()):
        self.bartab_user = bartab_user

    def __get_request_by_request_type(self, request_type: str, endpoint_name: str, data: dict, url_kwargs: dict):        
        if request_type == self.GET:
            request = self.factory.get(
                reverse(endpoint_name, kwargs=url_kwargs), data,  format=self.format)
        elif request_type == self.POST:
            request = self.factory.post(
                reverse(endpoint_name, kwargs=url_kwargs), data,  format=self.format)
        elif request_type == self.DELETE:
            request = self.factory.delete(
                reverse(endpoint_name, kwargs=url_kwargs), data,  format=self.format)
        elif request_type == self.PUT:
            request = self.factory.put(
                reverse(endpoint_name, kwargs=url_kwargs), data,  format=self.format)
        else:
            raise ValueError(f"Invalid request type {request_type}")

        request.resolver_match = ResolverMatch(None, None, url_kwargs)

        request.bartab_user = self.bartab_user

        return request


    def get_post_request(self, endpoint_name: str, data: dict = {}, url_kwargs: dict = {}):
        return self.__get_request_by_request_type(self.POST, endpoint_name, data, url_kwargs)
    
    def get_get_request(self, endpoint_name: str, data: dict = {}, url_kwargs: dict = {}):
        return self.__get_request_by_request_type(self.GET, endpoint_name, data, url_kwargs)

    def get_put_request(self, endpoint_name: str, data: dict = {}, url_kwargs: dict = {}):
        return self.__get_request_by_request_type(self.PUT, endpoint_name, data, url_kwargs)

    def get_delete_request(self, endpoint_name: str, data: dict = {}, url_kwargs: dict = {}):
        return self.__get_request_by_request_type(self.DELETE, endpoint_name, data, url_kwargs)


class MockMicroserviceResponse:
    def __init__(self, data):
        self.data = data

    def json(self):
        return self.data



import re
import shortuuid
from .string import split
from .integer import int_value_or_none


def valid_string_of_length(string, length: int):
    if string == None or not isinstance(string, str):
        return False
    
    length_value = int_value_or_none(length)
    
    if length_value == None:
        raise ValueError("length must be type int")
    else:
        return len(string) == length_value

def is_valid_uuid(uuid):
    if not valid_string_of_length(uuid, 36):
        return False
    else:
        regex = re.compile(
            "^[{]?[0-9a-fA-F]{8}" + "-([0-9a-fA-F]{4}-)" + "{3}[0-9a-fA-F]{12}[}]?$")

        return bool(re.search(regex, uuid))

def is_valid_short_uuid(uuid):
    if not valid_string_of_length(uuid, 22):
        return False
    else:
        valid_short_uuid_chars = shortuuid.get_alphabet()

        for uuid_char in split(uuid):
            if uuid_char not in valid_short_uuid_chars:
                return False
        return True


import jwt
from jwt import DecodeError
from jwt.algorithms import RSAAlgorithm
from django.http import HttpRequest
from rest_framework_jwt.settings import api_settings
from django.contrib.auth import authenticate


def get_username_from_payload_handler(payload) -> str:
    username = payload.get('sub')
    authenticate(remote_user=username)
    return username


def cognito_jwt_decode_handler(token: str) -> dict:
    """
    To verify the signature of an Amazon Cognito JWT, first search for the public key with a key ID that
    matches the key ID in the header of the token. (c)
    https://aws.amazon.com/premiumsupport/knowledge-center/decode-verify-cognito-json-token/
    Almost the same as default 'rest_framework_jwt.utils.jwt_decode_handler', but 'secret_key' feature is skipped
    """
    options = {'verify_exp': api_settings.JWT_VERIFY_EXPIRATION}
    unverified_header = jwt.get_unverified_header(token)
    if 'kid' not in unverified_header:
        raise DecodeError('Incorrect authentication credentials.')

    kid = unverified_header['kid']

    try:
        # pick a proper public key according to `kid` from token header
        public_key = RSAAlgorithm.from_jwk(api_settings.JWT_PUBLIC_KEY[kid])
    except KeyError:
        # in this place we could refresh cached jwks and try again
        raise DecodeError('Can\'t find proper public key in jwks')
    else:
        return jwt.decode(
            token,
            public_key,
            options=options,
            leeway=api_settings.JWT_LEEWAY,
            audience=api_settings.JWT_AUDIENCE,
            issuer=api_settings.JWT_ISSUER,
            algorithms=[api_settings.JWT_ALGORITHM]
        )


def get_bearer_token(request: HttpRequest) -> str:
    bearer_header = request.META.get('HTTP_AUTHORIZATION')
    if bearer_header != None and bearer_header != '':
        return request.META.get('HTTP_AUTHORIZATION').split(" ")[1]
    else:
        raise ValueError('No user authentication found')




from .list import normalize_list
from django.http import HttpRequest

class UrlParamsUtils:
    STATUS_URL_PARAM_NAME = "status"
    STATUS_CODE_URL_PARAM_NAME = "status_code"

    CANCELED_URL_PARAM_VALUE = "canceled"

    def __init__(self, request: HttpRequest):
        self.request = request

    def get_param_value(self, param):
        return self.request.GET.get(param, None)

    def get_status(self):
        return self.get_param_value(self.STATUS_URL_PARAM_NAME)

    def has_status(self):
        return self.has_param(self.STATUS_URL_PARAM_NAME)

    def is_cancel_request(self):
        return self.get_status() == self.CANCELED_URL_PARAM_VALUE

    def has_param(self, params):
        params = normalize_list(params)

        for param in params:
            if self.get_param_value(param) != None:
                return True
        return False

    def has_status_code(self):
        return self.has_param(self.STATUS_CODE_URL_PARAM_NAME)

    def get_status_code(self):
        if self.has_status_code():
            return self.get_param_value(self.STATUS_CODE_URL_PARAM_NAME).upper()
        else:
            return None



from datetime import time, datetime, timedelta
from django.conf import settings
from .date import TimeZoneData
import pytz
from .paths import get_relative_path
from .json import RequiredJsonFile
import time as pure_time

def current_milli_time():
    return round(pure_time.time() * 1000)


def get_hours_offset_from_float(float_number):
    return abs(int(float_number))


def get_minutes_offset_from_float(float_number: float) -> int:
    positive_modifier = 1 if float_number > 0 else -1

    hour_offset = get_hours_offset_from_float(float_number)

    float_minutes_str = str(abs(
        round(float_number - (hour_offset * positive_modifier), 2)))

    decimal_minutes_map_path = get_relative_path(
        './data/decimal_minutes_map.json', __file__)

    decimal_minutes_to_minutes_map = RequiredJsonFile(
        decimal_minutes_map_path).data()

    if float_minutes_str in decimal_minutes_to_minutes_map.keys():
        return decimal_minutes_to_minutes_map[float_minutes_str]
    else:
        raise ValueError(
            f"Minute decimal value: {float_minutes_str} not found ")


def get_offset_hours_from_float(float_number: float) -> str:
    hours = get_hours_offset_from_float(float_number)
    minutes = get_minutes_offset_from_float(float_number)

    sign = "+" if float_number >= 0 else '-'

    hours_display = hours if hours > 9 else "0{}".format(hours)
    minutes_display = minutes if minutes > 9 else "0{}".format(minutes)

    return "{sign}{h}{m}".format(sign=sign, h=hours_display, m=minutes_display)


def convert_datetime_to_time(date_time, date_time_format: str = None) -> time:
    if date_time_format == None:
        date_time_format = settings.DATETIME_FORMAT
    
    if isinstance(date_time, time):
        return date_time
    elif isinstance(date_time, str):
        date_time = datetime.strptime(date_time, date_time_format)
        return time(date_time.hour, date_time.minute)
    elif isinstance(date_time, datetime):
        return time(date_time.hour, date_time.minute)
    else:
        raise ValueError("Invalid date time format")


def time_in_range(begin_time, end_time, check_time=None) -> bool:
    check_time = datetime.utcnow().time() if check_time == None else convert_datetime_to_time(
        check_time)

    begin_time = convert_datetime_to_time(begin_time)
    end_time = convert_datetime_to_time(end_time)

    if begin_time < end_time:
        return check_time >= begin_time and check_time <= end_time
    else:  # crosses midnight
        return check_time >= begin_time or check_time <= end_time


def offset_hour(hour: int, hour_offset: int) -> int:
    # if adding time
    if hour_offset > 0:
        total = hour + hour_offset
        return total if total < 23 else total - 24
    # if removing time
    else:
        total = hour + hour_offset
        return total if total >= 0 else total + 24


def offset_minutes(minute: int, minutes_offset: int) -> int:
    total = minute + minutes_offset

    if total < 0:
        return total + 60
    elif total < 60:
        return total
    else:
        return total - 60


def convert_datetime_timezone(date_time: datetime, timezone: pytz.timezone) -> datetime:
    timezone_data = TimeZoneData.data()
    if str(date_time.tzinfo) in timezone_data['offset_by_timezone_name']:
        offset_float = float(
            timezone_data['offset_by_timezone_name'][str(date_time.tzinfo)]) * float(-1)
    else:
        offset_float = float(0.0) * float(-1)

    offset_h = get_hours_offset_from_float(offset_float)
    offset_min = get_minutes_offset_from_float(offset_float)

    utc_t = date_time + timedelta(hours=offset_h, minutes=offset_min)

    utc_time = datetime(utc_t.year, utc_t.month, utc_t.day,
                        utc_t.hour, utc_t.minute, utc_t.second, tzinfo=pytz.UTC)

    return utc_time.astimezone(timezone)



def get_pk_from_url(request) -> str:
    return request.resolver_match.kwargs.get('pk')


def get_named_id_from_url(request, name: str, allow_pk: bool = False) -> str:
    value = request.resolver_match.kwargs.get(name)

    if value != None:
        return value

    id_posfix = '_id'
    if len(name) < len(id_posfix) or name[len(name) - len(id_posfix):] != id_posfix:
        name = f'{name}{id_posfix}'
        value = request.resolver_match.kwargs.get(name)

    if value != None or (value == None and not allow_pk):
        return value

    return request.resolver_match.kwargs.get('pk')


def get_id_from_obj(obj) -> str:
    if isinstance(obj, str):
        return obj
    elif isinstance(obj, dict):
        if 'uuid' in obj:
            return obj['uuid']
        elif 'pk' in obj:
            return obj['pk']

    try:
        return obj.uuid
    except:
        return None



from .file import File
from .list import normalize_list
import json


def all_params_present(json_array: list, required_params: list) -> bool:
    required_params = normalize_list(required_params)
    json_array = normalize_list(json_array)

    for json_segment in json_array:
        if not isinstance(json_segment, dict):
            return False

        for required_value in required_params:
            if required_value not in json_segment:
                return False
    return True


def get_or_default(json_data, key, default):
    return default if key not in json_data else json_data[key]


def get_or_none(json_data, key):
    return get_or_default(json_data, key, None)


class JsonFile(File):
    def __init__(self, file_path: str, create_if_not_exists: bool = False):
        super().__init__(file_path, create_if_not_exists)

        if create_if_not_exists and not self.is_valid():
            self.set_json({})

    def is_valid(self):
        if not self.is_file_type('json'):
            return False

        try:
            json.loads(self.file_contents())
            return True
        except:
            return False

    def set_empty_json(self):
        self.set_json({})

    def set_json(self, json_data: dict):
        if not self.is_valid():
            self.get_or_create()
        json.dump(json_data, open(str(self.file_path), 'w'))

    def set_key(self, key: str, value):
        raw_data = self.data()
        raw_data[key] = value
        self.set_json(raw_data)

    def data(self):
        if self.is_valid():
            try:
                return json.load(self.read())
            except json.decoder.JSONDecodeError:
                self.set_json({})
                return self.data()
        else:
            raise ValueError("Invalid json file")


class RequiredJsonFile(JsonFile):
    def __init__(self, file_path):
        super().__init__(file_path)

        if not self.is_valid():
            raise ValueError("Invalid required json file")



import random
from .list import normalize_list
import re

DO_NOT_CAPITALIZE = ['A', 'THE', 'OF']

HAS_ONE_UPPERCASE_REGEX = '[A-Z]'

HAS_ONE_LOWERCASE_REGEX = '[a-z]'

HAS_ONE_NUMBER_REGEX = '[0-9]'

HAS_ONE_SPECIAL_CHARACTER = '[*@!#%&()^~}{]'

LOWER_CASE_LETTERS = 'abcdefghijklmnopqrstuzwxyz'
UPPERCASE_LETTERS = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
NUMBERS = '1234567890'
SPECIAL_CHARACTERS = "[*@!#%&()^~}{]"


def display(string: str) -> str:
    if string == None:
        return ''

    string_array = string.split(" ")
    formatted_string = string_array.pop(0).capitalize()
    for s in string_array:
        if s.upper() not in DO_NOT_CAPITALIZE:
            s = s.capitalize()
        else:
            s = s.lower()
        formatted_string += " "+s
    return formatted_string.strip()


def string_has_any_of(string: str, search_set: list) -> bool:
    for search_string in set(normalize_list(search_set)):
        if search_string in string:
            return True

    return False


def split(word: str) -> list:
    return [char for char in word]


def string_has_all_of(string: str, search_set: list) -> bool:
    for search_string in set(normalize_list(search_set)):
        if search_string not in string:
            return False

    return True


def get_clear_random_value(length: int = 8):
    letters = "ABCDEFGHJKLMNPQRSTUVWXYZ23456789"
    return ''.join(random.choice(letters) for i in range(length))


def string_has_all_properties(string, with_lower=True, with_upper=False, with_number=False, with_special=False):
    if with_lower and not string_has_any_of(string, split(LOWER_CASE_LETTERS)):
        return False

    if with_upper and not string_has_any_of(string, split(UPPERCASE_LETTERS)):
        return False

    if with_number and not string_has_any_of(string, split(NUMBERS)):
        return False

    if with_special and not string_has_any_of(string, split(SPECIAL_CHARACTERS)):
        return False

    return True


def get_random_string(
        length: int = 12,
        with_lower: bool = True,
        with_upper: bool = False,
        with_number: bool = False,
        with_special: bool = False,
        max_tires: int = 100000) -> str:
    if length < 1:
        raise ValueError("Length cannot be less then one")

    letters = ''
    minimum_length = 0

    if with_lower:
        letters += LOWER_CASE_LETTERS
        minimum_length = minimum_length + 1

    if with_upper:
        letters += UPPERCASE_LETTERS
        minimum_length = minimum_length + 1

    if with_number:
        letters += NUMBERS
        minimum_length = minimum_length + 1

    if with_special:
        letters += SPECIAL_CHARACTERS
        minimum_length = minimum_length + 1

    if len(letters) == 0:
        raise ValueError("Random string requires true requirement")
    elif length < minimum_length:
        raise ValueError(
            "Random string of those requirement and length cannot be generated")

    def get_random():
        return ''.join(random.choice(letters) for i in range(length))

    def has_all_required_params():
        return string_has_all_properties(
            possible_string,
            with_lower,
            with_upper,
            with_number,
            with_special
        )

    generation_tries = 1
    possible_string = get_random()
    while not has_all_required_params() and generation_tries < max_tires:
        generation_tries += 1
        possible_string = get_random()

    if has_all_required_params():
        return possible_string
    else:
        raise ValueError("String could not be successfully generated")


def camel_case_split(str: str) -> str:
    words = [[str[0]]]

    for c in str[1:]:
        if words[-1][-1].islower() and c.isupper():
            words.append(list(c))
        else:
            words[-1].append(c)

    return " ".join(map(lambda w: w.title(), [''.join(word) for word in words]))


def camel_case_to_slug(s: str) -> str:
    return camel_case_split(s).replace(" ", "_").lower()


def represents_int(s: str, fail_safely: str = True) -> bool:
    try:
        int(s)
        return True
    except ValueError:
        if fail_safely:
            return False
        raise ValueError


def validation_check(input_string: str, regex: str) -> bool:
    if re.search(regex, input_string):
        return True
    else:
        return False

def dash_case_to_upper_slug(s : str) -> str:
    if s == None:
        return None
    
    return s.upper().replace("-", '_')



from .microservices import AbstractMicroservice
from rest_framework.exceptions import APIException


class PainkillerUnavailable(APIException):
    status_code = 503
    default_detail = 'Order service temporarily unavailable, try again later.'
    default_code = 'service_unavailable'


class GreatPumpkinUnavailable(APIException):
    status_code = 503
    default_detail = 'Payment service temporarily unavailable, try again later.'
    default_code = 'service_unavailable'


class CementMixerUnavailable(APIException):
    status_code = 503
    default_detail = 'Location service temporarily unavailable, try again later.'
    default_code = 'service_unavailable'

class PainkillerClient(AbstractMicroservice):

    def __init__(self):
        super().__init__(
            url_prefix="pain",
            localhost_port="7777",
            service_name='painkiller',
            exception=PainkillerUnavailable,
            api_version="v1",
            service_description='order manager'
            )


class GreatPumpkinClient(AbstractMicroservice):
    def __init__(self):
        super().__init__(
            url_prefix="pumpkin",
            localhost_port="2344",
            service_name='great-pumpkin',
            exception=GreatPumpkinUnavailable,
            api_version="v1",
            service_description='payment manager'
        )


class CementMixerClient(AbstractMicroservice):

    def __init__(self):
        super().__init__(
            url_prefix="api",
            localhost_port="8000",
            service_name='cement-mixer',
            exception=CementMixerUnavailable,
            api_version="v1",
            service_description='location manager'
        )



import os
from .string import get_clear_random_value
import csv
import boto3
from .paths import get_relative_path


class File:

    def __init__(self, file_path: str, create_if_not_exists: bool = False):
        if file_path == None:
            raise ValueError("File path cannot be none")

        self.file_path = file_path

        file_split = file_path.split("/")
        self.file_name = file_split[-1]
        self.file_parent_dir = "/".join(file_split[:-1])

        if create_if_not_exists:
            self.get_or_create()

    def is_file_type(self, file_type: str):
        if not self.has_file_extention():
            return False

        return self.get_file_extention().lower() == file_type.lower()

    def exists(self, raise_error_on_non_exists: bool = False):
        try:
            exists = os.path.exists(self.file_path)
        except:
            exists = False

        if not exists and raise_error_on_non_exists:
            raise ValueError(
                f"File at file path {self.file_path} does not exist")

        return exists

    def has_file_extention(self, raise_error_on_invalid: bool = False):
        has_file_extention = "." in self.file_name

        if not has_file_extention and raise_error_on_invalid:
            raise ValueError(
                f"File at file path {self.file_path} is not valid")

        return has_file_extention

    def get_file_extention(self):
        self.has_file_extention(True)

        return self.file_name.split(".")[1]

    def get_file_name_no_extention(self):
        self.has_file_extention(True)

        return self.file_name.split(".")[0]

    def read(self):
        self.exists(True)

        return open(self.file_path, "r")

    def file_contents(self):
        self.exists(True)

        return open(self.file_path, "r").read()

    def replace(self, old: str, new: str, count: int = 0):
        if count > 0:
            self.set_text(self.file_contents().replace(
                old, new, count))
        else:
            self.set_text(self.file_contents().replace(
                old, new))

    def clear(self):
        self.exists(True)

        open(self.file_path, 'w').close()

    def allow_all(self):
        self.exists(True)

        os.chmod(self.file_path, 0o777)

    def delete(self):
        if os.path.exists(self.file_path):
            os.remove(self.file_path)

    def set_text(self, text: str):
        self.exists(True)

        file = open(self.file_path, "w")
        file.write(str(text))
        file.close()

    def append(self, text: str):
        self.exists(True)

        file = open(self.file_path, "a")
        file.write(text)
        file.close()

    def append_line(self, text: str):
        self.append(f"\n{text}")

    def get_or_create(self):
        if self.exists():
            return self
        else:
            if not os.path.exists(self.file_parent_dir):
                os.makedirs(self.file_parent_dir)

            open(self.file_path, "a")
            return self

    def __str__(self):
        return self.file_path


class RequiredFile(File):

    def __init__(self, file_path):
        super().__init__(file_path)
        if not super().exists():
            raise ValueError(
                f"File at file path {file_path} is not valid")


class FileTypeRequired(File):
    def __init__(self, file_path, file_type):
        super().__init__(file_path)
        if not super().exists() or not super().is_file_type(file_type):
            raise ValueError(
                f"File at file path {file_path} is not valid")


class TmpFile(File):
    def __init__(self, file_type="sh"):
        base_file_path = get_relative_path('../../tmp', __file__)

        file_path = f"{base_file_path}/{get_clear_random_value()}.{file_type}"

        super().__init__(file_path, True)

        self.clear()


class CSVFile(File):
    def __init__(self, file_path, delimiter=',', encoding="utf8", create_if_not_exists=False):
        super().__init__(file_path, create_if_not_exists)

        try:
            self.__rows = list(csv.reader(
                open(file_path, 'rt', encoding=encoding), delimiter=delimiter))
            self.is_valid = True
        except:
            self.is_valid = False

    def rows(self) -> list:
        return [] if not self.is_valid else self.__rows

    def headers(self) -> list:
        return [] if len(self.rows()) < 2 else self.rows()[0]

    def get_value(self, row: int, column: int):
        if not self.is_valid or len(self.rows()) <= row or len(self.rows()[row]) <= column:
            return None
        else:
            return self.rows()[row][column]

    def get_row(self, row: int) -> list:
        if not self.is_valid or len(self.rows()) <= row:
            return None
        else:
            return self.rows()[row]

    def find_row_by_value_in_column(self, column: int, value):
        if not self.is_valid:
            return None
        else:
            for row in self.rows():
                if len(row) > column and row[column] == value:
                    return row
            return None


class S3File(File):

    def __init__(self, bucket_name, bucket_file_path, file_path, raise_exception=False):
        super().__init__(file_path)

        self.bucket_name = bucket_name
        self.bucket_file_path = bucket_file_path
        self.raise_exception = raise_exception

        try:
            boto3.client('s3').download_file(bucket_name,
                                             bucket_file_path, file_path)
        except Exception as e:
            if raise_exception:
                raise ValueError("Error downloading file at file path {0} from bucket {1}".format(
                    bucket_file_path, bucket_name))
            else:
                self.file_path = None



def mock_address(
    address_one: str= "250 S Grand Ave",
    address_two: str = None,
    city: str = "Los Angeles",
    state: str = "CA",
    postal_code: str = "90012",
    country: str = 'US',
    lat: float = 34.05328815,
    lng: float = -118.25062277616152
) -> dict:
    address = {
        "address_one": address_one,
        "city": city,
        "state": state,
        "postal_code": postal_code,
        "country": country,
        "geocode": {
            "lat": lat,
            "lng": lng
        }
    }

    if address_two != None and address_two != '':
        address['address_two'] = address_two

    return address



from rest_framework.serializers import ValidationError

class AbstractDisplayValueValidator:
    def get_display(self, value_with_display):
        return value_with_display['display']

    def get_value(self, value_with_display):
        return value_with_display['value']

    def raise_error(self, value_with_display, error_messages):
        raise ValidationError(
            {self.get_display(value_with_display): error_messages})

    class Meta:
        abstract = True



from django.db import models
from shortuuidfield import ShortUUIDField
from django.db.models import CharField
from django.utils.translation import gettext_lazy as _
from .validate import is_valid_short_uuid, is_valid_uuid
from django.db import IntegrityError


class AbstractBaseModel(models.Model):
    """
    Base abstract model, that has `uuid` instead of `id` and includes `created_at`, `updated_at` fields.
    """
    uuid = ShortUUIDField(
        primary_key=True, editable=False, unique=True, max_length=22)
    created_at = models.DateTimeField('Created at', auto_now_add=True)
    updated_at = models.DateTimeField('Updated at', auto_now=True)

    class Meta:
        abstract = True

    def __repr__(self):
        return f'<{self.__class__.__name__} {self.uuid}>'


class UserField(CharField):
    description = 'User field'

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 36

        if 'null' not in kwargs:
            kwargs['null'] = False

        if 'blank' not in kwargs:
            kwargs['blank'] = False

        super().__init__(*args, **kwargs)

    def pre_save(self, model_instance, add: bool):
        value = super().pre_save(model_instance, add)

        if self.null and value == None:
            return value
        elif self.blank and value == '':
            return value
        elif not is_valid_uuid(value):
            raise IntegrityError("Invalid uuid")
        else:
            return value

class MicroserviceForginKey(CharField):
    description = _("Key used by different BarTab microservice")


    def __init__(self, *args, **kwargs):
        print(kwargs)
        kwargs['max_length'] = 22

        if 'null' not in kwargs:
            kwargs['null'] = False

        if 'blank' not in kwargs:
            kwargs['blank'] = False

        super().__init__(*args, **kwargs)

    def pre_save(self, model_instance, add: bool):
        value = super().pre_save(model_instance, add)

        if self.null and value == None:
            return value
        elif self.blank and value == '':
            return value
        elif not is_valid_short_uuid(value):
            raise IntegrityError("Invalid short uuid")
        else:
            return value



import os
from django.conf import settings

def get_domain() -> str:
    if settings.IS_PRODUCTION:
        return f"https://{settings.URL_PREFIX}.{settings.DOMAIN}"
    elif settings.IS_DEVELOPMENT:
        return f"https://dev.{settings.URL_PREFIX}.{settings.DOMAIN}"
    elif settings.IS_QA:
        return f"https://qa.{settings.URL_PREFIX}.{settings.DOMAIN}"
    else:
        return f"http://localhost:{settings.PORT}"


def get_parent_directory(file_running_script: str) -> str:
    file_running_script_array = file_running_script.split("/")

    dot_count = file_running_script_array[-1].count(".")
    if dot_count > 0:
        if file_running_script_array[-1][0] != '.' or dot_count > 1:
            del file_running_script_array[-1]
    
    return "/".join(file_running_script_array)

# Use __file__ to get path of file running script
def get_relative_path(path: str, file_running_script: str) -> str:
    starting_local_split = get_parent_directory(file_running_script).split("/")
    split_path = path.split('/')
    
    if split_path[0] == '.':
        if len(split_path) == 1:
            split_path = []
        else:
            split_path = split_path[1:]
    else:
        while len(split_path) > 0 and split_path[0] == "..":
            split_path.pop(0)
            starting_local_split = starting_local_split[:-1]

    starting_local = "/".join(starting_local_split)
    landing_path = "/".join(split_path)

    if landing_path == '':
        return starting_local
    else:
        return f"{starting_local}/{landing_path}"



from .validate import is_valid_uuid
from .user_info import UserInfoClient
class DenyAny:
    def has_permission(self, request, view):
        return False

    def has_object_permission(self, request, view, obj):
        return False


class PostWithoutAuth:
    def has_permission(self, request, view):
        return request.method == "POST"


class IsAuthenticated:
    def __is_authenticated(self, request):
        try:
            return is_valid_uuid(str(request.bartab_user))
        except:
            return False


    def has_permission(self, request, view):
        return self.__is_authenticated(request)

    def has_object_permission(self, request, view, obj):
        return self.__is_authenticated(request)


class IsAdminUser(IsAuthenticated):
    user_info_client = UserInfoClient()

    def has_permission(self, request, view):
        if not super().has_permission(request, view):
            return False

        return self.user_info_client.is_staff(str(request.bartab_user))

    def has_object_permission(self, request, view, obj):
        if not super().has_object_permission(request, view, obj):
            return False

        return self.user_info_client.is_staff(str(request.bartab_user))


class IsSalesUser(IsAuthenticated):
    user_info_client = UserInfoClient()

    def has_permission(self, request, view):
        if not super().has_permission(request, view):
            return False

        return self.user_info_client.is_sales(str(request.bartab_user))

    def has_object_permission(self, request, view, obj):
        if not super().has_object_permission(request, view, obj):
            return False

        return self.user_info_client.is_sales(str(request.bartab_user))



from setuptools import setup

setup(
    name='core_backend_bartab_utils',
    version='1.0.8',
    description='Common utils files used for any python3 BarTab repository',
    url='https://github.com/BarTabPayments/core_backend_bartab_utils',
    author='Cassio Hudson',
    author_email='cassio@bartabapp.co',
    license='BSD 2-clause',
    packages=['bartab_core'],
    install_requires=['mpi4py>=2.0',
                      'numpy',
                      ],

    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)



def normalize_list(value):
    if value == None:
        return []
    
    if type(value) is not list:
        return [value]
    else:
        return value


def comma_seperated_string_to_list(value: str):
    return list(map(lambda x: x.strip(), value.split(",")))


def valid_comma_seperated_list(values: str, valid_values: list):
    for value in comma_seperated_string_to_list(values):
        if value not in valid_values:
            return False
    return True



import environ
from django.conf import settings
from .paths import get_relative_path
from .string import dash_case_to_upper_slug

env = environ.Env()
environ.Env.read_env()


def get_api_key(service_name):
    return env("{}_API_KEY".format(
        dash_case_to_upper_slug(service_name)))


def get_invalid_configuration_error_message(service_name):
    api_key_param_name = "{}_API_KEY".format(
        dash_case_to_upper_slug(service_name))

    try:
        env(api_key_param_name)
        return None
    except:
        if settings.IS_DEVELOPMENT or settings.IS_PRODUCTION or settings.IS_QA:
            return "Missing {0} api key, run eb setenv {1}=<{1}>".format(
                service_name,
                api_key_param_name)
        else:
            return "Missing {0} api key, add {1} to: {2}".format(
                service_name,
                api_key_param_name,
                get_relative_path('.env', __file__))


def get_endpoint(url_prefix,
                 api_version,
                 localhost_port):
    if settings.IS_PRODUCTION:
        return "https://{url_prefix}.{DOMAIN}/{api_version}".format(
            url_prefix=url_prefix,
            DOMAIN=settings.DOMAIN,
            api_version=api_version
        )
    elif settings.IS_QA:
        return "https://qa.{url_prefix}.{DOMAIN}/{api_version}".format(
            url_prefix=url_prefix,
            DOMAIN=settings.DOMAIN,
            api_version=api_version
        )
    elif settings.IS_DEVELOPMENT:
        return "https://dev.{url_prefix}.{DOMAIN}/{api_version}".format(
            url_prefix=url_prefix,
            DOMAIN=settings.DOMAIN,
            api_version=api_version
        )
    else:
        return "http://localhost:{localhost_port}/{api_version}".format(
            localhost_port=localhost_port,
            api_version=api_version
        )
